@@ -38,8 +38,7 @@
 #include "bprefs.h"
 
 
-static const char RCSID[] = "$Id: backup.c,v 1.5 2006-09-27 19:15:27 rousseau Exp $";
-
+static const char RCSID[] = "$Id: backup.c,v 1.6 2009-02-22 08:32:16 rousseau Exp $";
 
 /* Local static functions */
 static int archive_dir_select (const struct dirent *entry);
@@ -364,6 +363,51 @@ int get_archive_file_name (const char *arch, const char *file, char *full_name,
     return 0;
 }
 
+int dbm_delete_items(char *name, GList *node) {
+    char dbf_name[256];
+    GDBM_FILE dbf;
+    datum key;
+    int ret;
+
+    /* open the src dbm file */
+    get_backup_file_name(name, dbf_name, 255);
+    dbf = gdbm_open(dbf_name, 512, GDBM_WRCREAT | LOCK_FLAG, 0644, 0);
+    if (!dbf) {
+        /* Can't open or create dbm file */
+        jp_logf (JP_LOG_FATAL,
+                 "Can't open dbm file %s\nReason: %s\n",
+                 dbf_name,
+                 gdbm_strerror (gdbm_errno));
+        return 1;
+    }
+
+    while (node) {
+        gchar *text = (gchar *)node->data;
+
+        jp_logf (JP_LOG_DEBUG,
+                     "dbm_delete_items() - item: %s, text: %s\n",
+                     dbf_name,
+                     text);
+
+        key.dptr = text;
+        key.dsize = strlen(text) + 1;
+
+        ret = gdbm_delete(dbf, key);
+        if (ret == -1) {
+            /* sanity check, shouldn't happen */
+            jp_logf (JP_LOG_WARN, "Key %s not found in dbm file!\n", text);
+        }
+
+        node = node->next;
+    }
+
+    /* close the dbm files */
+    gdbm_close(dbf);
+
+    return 0;
+}
+
+
 /*
  * Moves the given items from one DBM file to the other.
  */
@@ -401,7 +445,7 @@ int dbm_move_items (char *src_name, char *dest_name, GList *node) {
     }
 
     while (node) {
-        gchar *text = g_strdup((gchar *)node->data);
+        gchar *text = (gchar *)node->data;
 
         jp_logf (JP_LOG_DEBUG,
                      "dbm_move_items() - from: %s, to: %s, text: %s\n",
@@ -409,26 +453,25 @@ int dbm_move_items (char *src_name, char *dest_name, GList *node) {
                      dest_dbf_name,
                      text);
 
-        jp_charset_j2p(text, strlen (text) + 1);
         key.dptr = text;
         key.dsize = strlen (text) + 1;
 
         /* fetch the content */
         content = gdbm_fetch (src_dbf, key);
-        if (!content.dptr) {
+        if (content.dptr) {
+            /* store it in the dest dbm */
+            ret = gdbm_store (dest_dbf, key, content, GDBM_INSERT);
+
+            /* delete it from the src dbm */
+            ret = gdbm_delete (src_dbf, key);
+
+            free(content.dptr);
+        }
+        else {
             /* sanity check, shouldn't happen */
-            jp_logf (JP_LOG_WARN, "Key %s has no content!\n", text);
-            content.dptr = "0";
-            content.dsize = 2;
+            jp_logf (JP_LOG_FATAL, "Key %s not found in dbm file!\n", text);
         }
 
-        /* store it in the dest dbm */
-        ret = gdbm_store (dest_dbf, key, content, GDBM_INSERT);
-
-        /* delete it from the src dbm */
-        ret = gdbm_delete (src_dbf, key);
-
-	g_free(text);
         node = node->next;
     }
 
