@@ -30,35 +30,41 @@
 #include "bprefs.h"
 
 
-static const char RCSID[] = "$Id: gui.c,v 1.4 2006-09-16 09:21:41 rousseau Exp $";
+static const char RCSID[] = "$Id: gui.c,v 1.5 2009-02-22 08:32:16 rousseau Exp $";
 
+extern GtkTooltips *glob_tooltips;
 
 /* Global variables */
 static GtkWidget *active_clist;
 static GtkWidget *inactive_clist;
 static GtkWidget *backup_new_button;
+static GtkWidget *archive_spinner;
+static GtkWidget *persistent_archive_button;
 static GtkWidget *every_sync_button;
 static GtkWidget *daily_button;
 static GtkWidget *weekly_button;
 static GtkWidget *monthly_button;
-static GtkWidget *archive_spinner;
-static GtkWidget *persistent_archive_button;
-
 
 /* Local static functions */
 static void update_prefs_gui (void);
 static void move_selected_item (GtkCList *src, GtkCList *dest);
-static void move_all_items (GtkCList *src, GtkCList *dest);
-static void cb_move_all_to_inactive (GtkWidget *widget, int data);
-static void cb_move_to_inactive (GtkWidget *widget, int data);
-static void cb_move_all_to_active (GtkWidget *widget, int data);
-static void cb_move_to_active (GtkWidget *widget, int data);
+static void delete_selected_items(GtkCList *clist);
+static void load_clist(GDBM_FILE dbf, GtkCList *clist);
+static void cb_move_to_inactive (GtkWidget *widget, gpointer data);
+static void cb_move_to_active (GtkWidget *widget, gpointer data);
+static void cb_delete_selected_items(GtkWidget *widget, gpointer data);
 static void cb_toggle_button (GtkWidget *widget, gpointer data);
 static void cb_archives (GtkWidget *widget, gpointer data);
+#ifdef ENABLE_GTK2
+static gboolean cb_popup(GtkWidget *widget, GdkEventButton *event, gpointer data);
+static void show_popup(GtkWidget *treeview, GdkEventButton *event, gpointer userdata);
+#else
+static gboolean cb_popup(GtkWidget *widget, gpointer data);
+#endif
 
 
 int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
-    GtkWidget *vbox1, *vbox2, *vbox0;
+    GtkWidget *vbox1, *vbox2, *vbox0, *button_vbox;
     GtkWidget *temp_hbox;
     GtkWidget *button;
     GtkWidget *label;
@@ -66,11 +72,11 @@ int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
     GtkWidget *right_scrolled_window;
     GtkObject *adj;
 
-
     /* left and right main boxes */
     vbox1 = gtk_vbox_new(FALSE, 0);
     vbox2 = gtk_vbox_new(FALSE, 0);
     vbox0 = gtk_vbox_new(FALSE, 0);
+    button_vbox = gtk_vbox_new(FALSE, 0);
 
     /* ----------------------------------- */
     /* Top portion of screen (preferences) */
@@ -125,23 +131,23 @@ int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
                         GTK_SIGNAL_FUNC (cb_toggle_button),
                         GINT_TO_POINTER (EVERY_SYNC_BUTTON));
     gtk_box_pack_start (GTK_BOX (temp_hbox), every_sync_button, TRUE, FALSE, 0);
+
     daily_button = gtk_radio_button_new_with_label(
-        gtk_radio_button_group (GTK_RADIO_BUTTON (every_sync_button)),
-        "Daily");
+        gtk_radio_button_group (GTK_RADIO_BUTTON (every_sync_button)), "Daily");
     gtk_signal_connect (GTK_OBJECT (daily_button), "toggled",
                         GTK_SIGNAL_FUNC (cb_toggle_button),
                         GINT_TO_POINTER (DAILY_BUTTON));
     gtk_box_pack_start (GTK_BOX (temp_hbox), daily_button, TRUE, FALSE, 0);
+
     weekly_button = gtk_radio_button_new_with_label(
-        gtk_radio_button_group (GTK_RADIO_BUTTON (daily_button)),
-        "Weekly");
+        gtk_radio_button_group (GTK_RADIO_BUTTON (daily_button)), "Weekly");
     gtk_signal_connect (GTK_OBJECT (weekly_button), "toggled",
                         GTK_SIGNAL_FUNC (cb_toggle_button),
                         GINT_TO_POINTER (WEEKLY_BUTTON));
     gtk_box_pack_start (GTK_BOX (temp_hbox), weekly_button, TRUE, FALSE, 0);
+
     monthly_button = gtk_radio_button_new_with_label(
-        gtk_radio_button_group (GTK_RADIO_BUTTON (weekly_button)),
-        "Monthly");
+        gtk_radio_button_group (GTK_RADIO_BUTTON (weekly_button)), "Monthly");
     gtk_signal_connect (GTK_OBJECT (monthly_button), "toggled",
                         GTK_SIGNAL_FUNC (cb_toggle_button),
                         GINT_TO_POINTER (MONTHLY_BUTTON));
@@ -152,6 +158,7 @@ int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
 
     temp_hbox = gtk_hbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX(temp_hbox), vbox1, TRUE, TRUE, 5);
+    gtk_box_pack_start (GTK_BOX(temp_hbox), button_vbox, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX(temp_hbox), vbox2, TRUE, TRUE, 5);
     gtk_box_pack_start (GTK_BOX(vbox0), temp_hbox, TRUE, TRUE, 5);
     gtk_box_pack_start (GTK_BOX(hbox), vbox0, TRUE, TRUE, 5);
@@ -183,25 +190,32 @@ int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
 
     /* Active Clist */
     active_clist = gtk_clist_new(1);
-    gtk_clist_set_selection_mode(GTK_CLIST(active_clist), GTK_SELECTION_SINGLE);
+    gtk_clist_set_selection_mode(GTK_CLIST(active_clist), GTK_SELECTION_MULTIPLE);
     gtk_clist_set_column_width(GTK_CLIST(active_clist), 0, 150);
     gtk_clist_set_auto_sort (GTK_CLIST(active_clist), TRUE);
     gtk_container_add(GTK_CONTAINER(left_scrolled_window), GTK_WIDGET(active_clist));
     gtk_object_set_user_data (GTK_OBJECT(active_clist), (gpointer)ACTIVE_DBM);
+    gtk_tooltips_set_tip(glob_tooltips, active_clist, "Right-click for options", NULL);
+#ifdef ENABLE_GTK2
+    g_signal_connect(active_clist, "button_press_event", G_CALLBACK(cb_popup), NULL);
+#else
 
-    temp_hbox = gtk_hbox_new(FALSE, 0);
+#endif
 
-    button = gtk_button_new_with_label("Move All >>");
-    gtk_box_pack_start(GTK_BOX(temp_hbox), button, TRUE, FALSE, 10);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(cb_move_all_to_inactive), NULL);
-
-    button = gtk_button_new_with_label("Move >>");
-    gtk_box_pack_start(GTK_BOX(temp_hbox), button, TRUE, FALSE, 10);
+    /* Move Buttons */
+    button = gtk_button_new_with_label("Ignore -->");
+    gtk_box_pack_start(GTK_BOX(button_vbox), button, TRUE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(cb_move_to_inactive), NULL);
+    gtk_tooltips_set_tip(glob_tooltips, button, "Move selected items to the ignore list", NULL);
 
-    gtk_box_pack_start (GTK_BOX(vbox1), temp_hbox, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX(button_vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
+
+    button = gtk_button_new_with_label("<-- Backup");
+    gtk_box_pack_start(GTK_BOX(button_vbox), button, TRUE, FALSE, 0);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                       GTK_SIGNAL_FUNC(cb_move_to_active), NULL);
+    gtk_tooltips_set_tip(glob_tooltips, button, "Move selected items to the active list", NULL);
 
     /* -------------------- */
     /* Right half of screen */
@@ -224,25 +238,17 @@ int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
 
     /* Inactive Clist */
     inactive_clist = gtk_clist_new(1);
-    gtk_clist_set_selection_mode(GTK_CLIST(inactive_clist), GTK_SELECTION_SINGLE);
+    gtk_clist_set_selection_mode(GTK_CLIST(inactive_clist), GTK_SELECTION_MULTIPLE);
     gtk_clist_set_column_width(GTK_CLIST(inactive_clist), 0, 150);
     gtk_clist_set_auto_sort (GTK_CLIST(inactive_clist), TRUE);
     gtk_container_add(GTK_CONTAINER(right_scrolled_window), GTK_WIDGET(inactive_clist));
     gtk_object_set_user_data (GTK_OBJECT(inactive_clist), (gpointer)INACTIVE_DBM);
+    gtk_tooltips_set_tip(glob_tooltips, inactive_clist, "Right-click for options", NULL);
+#ifdef ENABLE_GTK2
+    g_signal_connect(inactive_clist, "button_press_event", G_CALLBACK(cb_popup), NULL);
+#else
 
-    temp_hbox = gtk_hbox_new(FALSE, 0);
-
-    button = gtk_button_new_with_label("<< Move");
-    gtk_box_pack_start(GTK_BOX(temp_hbox), button, TRUE, FALSE, 10);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(cb_move_to_active), NULL);
-
-    button = gtk_button_new_with_label("<< Move All");
-    gtk_box_pack_start(GTK_BOX(temp_hbox), button, TRUE, FALSE, 10);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                       GTK_SIGNAL_FUNC(cb_move_all_to_active), NULL);
-
-    gtk_box_pack_start (GTK_BOX(vbox2), temp_hbox, FALSE, FALSE, 0);
+#endif
 
     gtk_widget_show_all(hbox);
     gtk_widget_show_all(vbox);
@@ -254,13 +260,12 @@ int init_gui (GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id) {
 }
 
 int destroy_gui() {
-    gtk_clist_clear (GTK_CLIST(active_clist));
-    gtk_clist_clear (GTK_CLIST(inactive_clist));
-
-    if (GTK_IS_WIDGET(active_clist)) {
+    if (GTK_IS_CLIST(active_clist)) {
+        gtk_clist_clear (GTK_CLIST(active_clist));
         gtk_widget_destroy (active_clist);
     }
-    if (GTK_IS_WIDGET(inactive_clist)) {
+    if (GTK_IS_CLIST(inactive_clist)) {
+        gtk_clist_clear (GTK_CLIST(inactive_clist));
         gtk_widget_destroy (inactive_clist);
     }
 
@@ -270,7 +275,41 @@ int destroy_gui() {
     return 0;
 }
 
+#ifdef ENABLE_GTK2
+static void show_popup(GtkWidget *widget, GdkEventButton *event, gpointer data) {
+    GtkWidget *menu, *menuitem;
 
+    menu = gtk_menu_new();
+
+    if (widget == active_clist) {
+        menuitem = gtk_menu_item_new_with_label("Ignore Selected Items");
+        g_signal_connect(menuitem, "activate", (GCallback) cb_move_to_inactive, widget);
+    }
+    else if (widget == inactive_clist) {
+        menuitem = gtk_menu_item_new_with_label("Backup Selected Items");
+        g_signal_connect(menuitem, "activate", (GCallback) cb_move_to_active, widget);
+    }
+    else {
+        jp_logf (JP_LOG_FATAL, "%s\n" "Invalid widget passed");
+        return;
+    }
+
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+    menuitem = gtk_menu_item_new_with_label("Remove Selected Items");
+    g_signal_connect(menuitem, "activate", (GCallback) cb_delete_selected_items, widget);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+    gtk_widget_show_all(menu);
+
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                   (event != NULL) ? event->button : 0,
+                   gdk_event_get_time((GdkEvent*)event));
+}
+
+#else
+
+#endif
 
 static void update_prefs_gui() {
     long ivalue;
@@ -287,7 +326,7 @@ static void update_prefs_gui() {
     backup_get_pref (BPREF_BACKUP_WHEN, &ivalue, NULL);
     switch (ivalue) {
         case EVERY_SYNC:
-            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (backup_new_button), TRUE);
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (every_sync_button), TRUE);
             break;
 
         case DAILY:
@@ -315,10 +354,8 @@ static void update_prefs_gui() {
 int display_databases() {
     char full_name[256];
     GDBM_FILE dbf;
-    datum key, nextkey;
 
     jp_logf (JP_LOG_DEBUG, "Backup: display databases\n");
-#define CHARSET_CONVERT_BUFFER 1024
 
     /*
      * Let's make sure the CLists have been created before we go and
@@ -339,74 +376,17 @@ int display_databases() {
     get_backup_file_name (ACTIVE_DBM, full_name, 255);
     dbf = gdbm_open (full_name, 512, GDBM_READER | LOCK_FLAG, 0644, 0);
     if (dbf) {
-        /* load the active databases */
-        key = gdbm_firstkey (dbf);
-        if (key.dptr) {
-	    gchar *utf8_text = g_malloc(CHARSET_CONVERT_BUFFER);
-            jp_logf (JP_LOG_DEBUG, "Retrieved %s from active database file\n", key.dptr);
-	    g_strlcpy(utf8_text, key.dptr, CHARSET_CONVERT_BUFFER);
-	    jp_charset_p2j(utf8_text, CHARSET_CONVERT_BUFFER);
-	    gtk_clist_append (GTK_CLIST(active_clist), &utf8_text);
-
-            while (key.dptr) {
-                nextkey = gdbm_nextkey (dbf, key);
-                if (nextkey.dptr) {
-                    jp_logf (JP_LOG_DEBUG, "Retrieved %s from active database file\n", nextkey.dptr);
-		    g_strlcpy(utf8_text, nextkey.dptr, CHARSET_CONVERT_BUFFER);
-		    jp_charset_p2j(utf8_text, CHARSET_CONVERT_BUFFER);
-		    gtk_clist_append (GTK_CLIST(active_clist), &utf8_text);
-                }
-                free (key.dptr);
-                key = nextkey;
-            }
-	    g_free(utf8_text);
-        }
-        else {
-            jp_logf (JP_LOG_GUI, "No active databases found\n");
-        }
-
+        load_clist(dbf, GTK_CLIST(active_clist));
         gdbm_close (dbf);
     }
-    else {
-        jp_logf (JP_LOG_DEBUG, "Can't open dbm file %s\n", full_name);
-    }
-
 
     /* open the inactive dbm file */
     get_backup_file_name (INACTIVE_DBM, full_name, 255);
     dbf = gdbm_open (full_name, 512, GDBM_READER | LOCK_FLAG, 0644, 0);
     if (dbf) {
-        /* load the inactive databases */
-        key = gdbm_firstkey (dbf);
-        if (key.dptr) {
-	    char *utf8_text = g_malloc(CHARSET_CONVERT_BUFFER);
-            jp_logf (JP_LOG_DEBUG, "Retrieved %s from inactive database file\n", key.dptr);
-	    g_strlcpy(utf8_text, key.dptr, CHARSET_CONVERT_BUFFER);
-	    jp_charset_p2j(utf8_text, CHARSET_CONVERT_BUFFER);
-            gtk_clist_append (GTK_CLIST(inactive_clist), &utf8_text);
-
-            while (key.dptr) {
-                nextkey = gdbm_nextkey (dbf, key);
-                if (nextkey.dptr) {
-                    jp_logf (JP_LOG_DEBUG, "Retrieved %s from inactive database file\n", nextkey.dptr);
-		    g_strlcpy(utf8_text, nextkey.dptr, CHARSET_CONVERT_BUFFER);
-		    jp_charset_p2j(utf8_text, CHARSET_CONVERT_BUFFER);
-                    gtk_clist_append (GTK_CLIST(inactive_clist), &utf8_text);
-                }
-                free (key.dptr);
-                key = nextkey;
-            }
-	    g_free(utf8_text);
-        }
-        else {
-            jp_logf (JP_LOG_GUI, "No inactive databases found\n");
-        }
-
+        load_clist(dbf, GTK_CLIST(inactive_clist));
         gdbm_close (dbf);
     }
-    else {
-        jp_logf (JP_LOG_DEBUG, "Can't open dbm file %s\n", full_name);
-    }
 
     gtk_clist_unselect_all (GTK_CLIST(active_clist));
     gtk_clist_unselect_all (GTK_CLIST(inactive_clist));
@@ -417,95 +397,189 @@ int display_databases() {
     return 0;
 }
 
+/*
+ * Loads the CList from the dbm file.
+ */
+static void load_clist(GDBM_FILE dbf, GtkCList *clist) {
+    datum key, nextkey;
+    datum content;
+    gint row;
+    char *utf8_text = g_malloc(CHARSET_CONVERT_BUFFER);
+
+    key = gdbm_firstkey (dbf);
+    while (key.dptr) {
+        jp_logf (JP_LOG_DEBUG, "Retrieved %s from database file\n", key.dptr);
+        g_strlcpy(utf8_text, key.dptr, CHARSET_CONVERT_BUFFER);
+        jp_charset_p2j(utf8_text, CHARSET_CONVERT_BUFFER);
+        row = gtk_clist_append (clist, &utf8_text);
+        gtk_clist_set_row_data_full(clist, row, g_strdup(key.dptr), (GtkDestroyNotify)g_free);
+
+        content = gdbm_fetch(dbf, key);
+        if (content.dptr) {
+            time_t mtime = (time_t)atoi(content.dptr);
+            jp_logf(JP_LOG_DEBUG, "mtime: %ld\n", mtime);
+
+            /*
+             * If the mtime of the database file is 0, then it is no longer on
+             * the handheld.  In that case, change the style of the entry in
+             * the clist to the stock deleted style.
+             */
+            if (mtime == 0) {
+                GtkStyle *style = gtk_clist_get_row_style(clist, row);
+
+                if (style) {
+                    jp_logf(JP_LOG_DEBUG, "%s\n", "found style");
+                    style = gtk_style_copy(style);
+                }
+                else {
+                    jp_logf(JP_LOG_DEBUG, "%s\n", "style not found");
+                    style = gtk_style_new();
+                }
+
+                style->base[GTK_STATE_NORMAL].red = CLIST_DEL_RED;
+                style->base[GTK_STATE_NORMAL].green = CLIST_DEL_GREEN;
+                style->base[GTK_STATE_NORMAL].blue = CLIST_DEL_BLUE;
+                style->fg[GTK_STATE_SELECTED].red = CLIST_DEL_RED;
+                style->fg[GTK_STATE_SELECTED].green = CLIST_DEL_GREEN;
+                style->fg[GTK_STATE_SELECTED].blue = CLIST_DEL_BLUE;
+                gtk_clist_set_row_style(clist, row, style);
+            }
+
+            free(content.dptr);
+        }
+
+        nextkey = gdbm_nextkey(dbf, key);
+        free(key.dptr);
+        key = nextkey;
+    }
+
+    g_free(utf8_text);
+}
 
 /*
  * Moves the selected item in the from CList to the to CList.
  */
 static void move_selected_item (GtkCList *src, GtkCList *dest) {
     guint row;
-    GList *node;
+    gint dest_row;
+    GList *selection;
+    GList *items = NULL;
     gchar *text;
     gpointer src_name;
     gpointer dest_name;
+    char *dbname;
+    GtkStyle *style;
+
+    /* Sanity check */
+    if (!src->selection) return;
 
     src_name = gtk_object_get_user_data (GTK_OBJECT(src));
     dest_name = gtk_object_get_user_data (GTK_OBJECT(dest));
 
-    node = src->selection;
-    if (node) {
-        row = GPOINTER_TO_UINT(node->data);
-        gtk_clist_get_text (src, row, 0, &text);
-        node = NULL;
-        node = g_list_append (node, text);
-        dbm_move_items (src_name, dest_name, node);
-        g_list_free (node);
-        gtk_clist_append (dest, &text);
-        gtk_clist_remove (src, row);
-    }
-}
-
-/*
- * Moves all items in the src CList to the dest CList.
- */
-static void move_all_items (GtkCList *src, GtkCList *dest) {
-    gchar *text;
-    gpointer src_name;
-    gpointer dest_name;
-    GList *list = NULL;
-    int i;
-
-    /*
-     * Get the DBM filenames from the CList user data.
-     */
-    src_name = gtk_object_get_user_data (GTK_OBJECT(src));
-    dest_name = gtk_object_get_user_data (GTK_OBJECT(dest));
-
-    /*
-     * Freeze the CLists, so we don't get an annoying flicker.
-     */
     gtk_clist_freeze (src);
     gtk_clist_freeze (dest);
 
-    /*
-     * Iterate over each of the items in the src...
-     */
-    for (i = 0; gtk_clist_get_text (src, i, 0, &text); i++) {
-        /* Append the item to the dest CList and store it in the GList */
-        gtk_clist_append (dest, &text);
-        list = g_list_append (list, text);
+    /* Iterate over the list backwards, so the row numbers don't change. */
+    while ((selection = g_list_last(src->selection)) != NULL) {
+        row = GPOINTER_TO_UINT(selection->data);
+        gtk_clist_get_text (src, row, 0, &text);
+        dbname = g_strdup((char *)gtk_clist_get_row_data(src, row));
+
+        jp_logf(JP_LOG_DEBUG, "text: %s\n", text);
+        jp_logf(JP_LOG_DEBUG, "data: %s\n", dbname);
+
+        style = gtk_clist_get_row_style(src, row);
+        items = g_list_append(items, dbname);
+        dest_row = gtk_clist_append (dest, &text);
+        gtk_clist_set_row_data(dest, dest_row, dbname);
+        gtk_clist_remove (src, row);
+
+        if (style != NULL) {
+            jp_logf(JP_LOG_DEBUG, "%s\n", "found style");
+            gtk_clist_set_row_style(dest, dest_row, gtk_style_copy(style));
+        }
+        else {
+            jp_logf(JP_LOG_DEBUG, "%s\n", "style NOT found :-(");
+        }
     }
 
-    /* Move the items in the DBM files */
-    dbm_move_items (src_name, dest_name, list);
-    g_list_free (list);
+    dbm_move_items(src_name, dest_name, items);
+    g_list_free(items);
 
-    gtk_clist_clear (src);
     gtk_clist_thaw (src);
     gtk_clist_thaw (dest);
 }
 
+static void delete_selected_items(GtkCList *clist) {
+    guint row;
+    GList *selection;
+    GList *items = NULL;
+    gchar *text;
+    gpointer name;
+    char *dbname;
+
+    jp_logf(JP_LOG_DEBUG, "%s\n", "delete");
+
+    /* Sanity check */
+    if (!clist->selection) return;
+
+    name = gtk_object_get_user_data(GTK_OBJECT(clist));
+    jp_logf(JP_LOG_DEBUG, "name: %s\n", name);
+    gtk_clist_freeze(clist);
+
+    /* Iterate over the list backwards, so the row numbers don't change. */
+    while ((selection = g_list_last(clist->selection)) != NULL) {
+        row = GPOINTER_TO_UINT(selection->data);
+        gtk_clist_get_text(clist, row, 0, &text);
+        dbname = g_strdup((char *)gtk_clist_get_row_data(clist, row));
+
+        jp_logf(JP_LOG_DEBUG, "text: %s\n", text);
+        jp_logf(JP_LOG_DEBUG, "data: %s\n", dbname);
+
+        items = g_list_append(items, dbname);
+        gtk_clist_remove(clist, row);
+    }
+
+    dbm_delete_items(name, items);
+    g_list_free(items);
+    gtk_clist_thaw(clist);
+}
 
 
 /* ----------------------------------------------------------------------- */
 /*                               Callbacks                                 */
 /* ----------------------------------------------------------------------- */
 
-static void cb_move_all_to_inactive (GtkWidget *widget, int data) {
-    move_all_items (GTK_CLIST(active_clist), GTK_CLIST(inactive_clist));
-}
-
-static void cb_move_to_inactive (GtkWidget *widget, int data) {
+static void cb_move_to_inactive (GtkWidget *widget, gpointer data) {
     move_selected_item (GTK_CLIST(active_clist), GTK_CLIST(inactive_clist));
 }
 
-static void cb_move_all_to_active (GtkWidget *widget, int data) {
-    move_all_items (GTK_CLIST(inactive_clist), GTK_CLIST(active_clist));
-}
-
-static void cb_move_to_active (GtkWidget *widget, int data) {
+static void cb_move_to_active (GtkWidget *widget, gpointer data) {
     move_selected_item (GTK_CLIST(inactive_clist), GTK_CLIST(active_clist));
 }
 
+static void cb_delete_selected_items(GtkWidget *widget, gpointer data) {
+    delete_selected_items(GTK_CLIST(data));
+}
+
+#ifdef ENABLE_GTK2
+static gboolean cb_popup(GtkWidget *widget, GdkEventButton *event, gpointer data) {
+    GdkEventButton *bevent = (GdkEventButton *)event;
+
+    if (event->type == GDK_BUTTON_PRESS && bevent->button == 3) {
+        show_popup(widget, event, data);
+    }
+
+    return FALSE;
+}
+
+#else
+static gboolean cb_popup(GtkWidget *widget, gpointer data) {
+    printf("boo\n");
+    return FALSE;
+}
+#endif
+
 static void cb_toggle_button (GtkWidget *widget, gpointer data) {
     switch (GPOINTER_TO_INT (data)) {
         case BACKUP_NEW_BUTTON:
