@@ -1,6 +1,6 @@
 /* plugin.c
  *
- * Copyright (C) 2002 by Jason Day
+ * Copyright (C) 2002-2006 by Jason Day
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <gdbm.h>
 #include <dirent.h>
 #include <errno.h>
+#include <glib.h>
 
 #include <pi-dlp.h>
 #include <pi-file.h>
@@ -41,12 +42,12 @@
 #include "bprefs.h"
 
 
-static const char RCSID[] = "$Id: plugin.c,v 1.5 2006-09-16 09:21:41 rousseau Exp $";
-
-
+static const char RCSID[] = "$Id: plugin.c,v 1.6 2009-02-22 08:32:16 rousseau Exp $";
 
 /* Local static functions */
 static void filename_make_legal(char *s);
+static void purge_db(GDBM_FILE dbf, GHashTable *hashtable);
+static gboolean rm_func(gpointer key, gpointer value, gpointer user_data);
 
 
 /* This plugin was designed to work with version 0.99 or greater of jpilot */
@@ -74,9 +75,9 @@ int plugin_help (char **text, int *width, int *height) {
    *text = strdup(
 	   "Backup plugin for J-Pilot\n"
            "version " VERSION "\n"
-	   "by Jason Day (c) 1999-2002.\n"
-	   "jasonday@worldnet.att.net\n"
-	   "http://jasonday.home.att.net/\n"
+           "by Jason Day (c) 1999-2008.\n"
+	   "jason@jlogday.com\n"
+	   "http://www.jlogday.com/\n"
 	   );
 
    /* Specifying 0 for width and height lets GTK decide */
@@ -115,7 +116,7 @@ int plugin_sync (int sd) {
     int start;
     struct DBInfo info;
 #ifdef PILOT_LINK_0_12
-        pi_buffer_t *buffer;
+    pi_buffer_t *buffer;
 #endif
     time_t ltime;
     struct tm *now;
@@ -133,6 +134,7 @@ int plugin_sync (int sd) {
     FILE *manifest;
     long backup_new;
     long persistent_archive;
+    GHashTable *hashtable;
 
 
     /* see if it's time to make a backup */
@@ -196,6 +198,8 @@ int plugin_sync (int sd) {
     backup_get_pref (BPREF_BACKUP_NEW, &backup_new, NULL);
     backup_get_pref (BPREF_PERSISTENT_ARCHIVE, &persistent_archive, NULL);
 
+    hashtable = g_hash_table_new(g_str_hash, g_int_equal);
+
     start = 0;
 #ifdef PILOT_LINK_0_12
     buffer = pi_buffer_new(32 * sizeof(struct DBInfo));
@@ -212,9 +216,29 @@ int plugin_sync (int sd) {
 
         key.dptr = info.name;
         key.dsize = strlen (info.name) + 1;
+        g_hash_table_insert(hashtable, g_strdup(info.name), GINT_TO_POINTER(1));
 
         /* see if it's in the inactive list */
-        if (gdbm_exists (inactive_dbf, key)) {
+        /*if (gdbm_exists (inactive_dbf, key)) {*/
+        content = gdbm_fetch (inactive_dbf, key);
+        if (content.dptr) {
+            mtime = (time_t)atoi (content.dptr);
+            free(content.dptr);
+
+            /*
+             * If the mtime of this database file is currently 0 in the dbf,
+             * that means it was removed at some time from the handheld but is
+             * now present again.  So update the dbm record with the current
+             * mtime.
+             */
+            if (mtime == 0) {
+                sprintf (temp_str, "%ld", info.modifyDate);
+                content.dptr = temp_str;
+                content.dsize = strlen (temp_str) + 1;
+                ret = gdbm_store (inactive_dbf, key, content, GDBM_REPLACE);
+                jp_logf (JP_LOG_DEBUG, "Updating mtime of %s in inactive database file\n", key.dptr);
+            }
+
             continue;
         }
 
@@ -224,6 +248,7 @@ int plugin_sync (int sd) {
         content = gdbm_fetch (active_dbf, key);
         if (content.dptr) {
             mtime = (time_t)atoi (content.dptr);
+            free(content.dptr);
         }
         else {
             /*
@@ -329,8 +354,16 @@ int plugin_sync (int sd) {
     pi_buffer_free(buffer);
 #endif
 
+    /* set mtime to 0 for databases which are no longer on the handheld */
+    purge_db(active_dbf, hashtable);
+    purge_db(inactive_dbf, hashtable);
 
-    /* close the files */
+    /* free the hashtable */
+    g_hash_table_freeze(hashtable);
+    g_hash_table_foreach_remove(hashtable, (GHRFunc)rm_func, NULL);
+    g_hash_table_destroy(hashtable);
+
+    /* close the dbm files */
     gdbm_close (active_dbf);
     gdbm_close (inactive_dbf);
     if (manifest) {
@@ -347,6 +380,39 @@ int plugin_sync (int sd) {
     return 0;
 }
 
+static gboolean rm_func(gpointer key, gpointer value, gpointer user_data) {
+    g_free(key);
+    return TRUE;
+}
+
+/*
+ * Iterates over each entry in the dbm file and checks whether that entry
+ * exists in the hashtable.  If an entry in the dbm does not exist in the
+ * hashtable, then its data (representing the mtime of the database file) is
+ * set to zero.
+ */
+static void purge_db(GDBM_FILE dbf, GHashTable *hashtable) {
+    datum key, nextkey;
+    datum content;
+    char *str = "0";
+
+    key = gdbm_firstkey (dbf);
+    while (key.dptr) {
+        jp_logf (JP_LOG_DEBUG, "Retrieved %s from database file\n", key.dptr);
+
+        if (!g_hash_table_lookup(hashtable, key.dptr)) {
+            /* Not in hashtable, set the mtime to 0 */
+            content.dptr = str;
+            content.dsize = strlen (str) + 1;
+            gdbm_store (dbf, key, content, GDBM_REPLACE);
+        }
+
+        nextkey = gdbm_nextkey (dbf, key);
+        free (key.dptr);
+        key = nextkey;
+    }
+}
+
 int plugin_post_sync() {
     jp_logf (JP_LOG_DEBUG, "Backup: plugin_post_sync\n");
     /* For some reason, calling this will cause the sync process to die
